<?php
namespace CloudPrinterPlugin;

use CloudPrinterPlugin\Controllers\CheckoutController;
use CloudPrinterPlugin\Controllers\FixNonceController;
use CloudPrinterPlugin\Controllers\GeneralSettingsController;
use CloudPrinterPlugin\Controllers\InstallPluginController;
use CloudPrinterPlugin\Controllers\ManualProductFileUploadController;
use CloudPrinterPlugin\Controllers\OrderAdminController;
use CloudPrinterPlugin\Controllers\OrderCancelController;
use CloudPrinterPlugin\Controllers\OrderDetailController;
use CloudPrinterPlugin\Controllers\OrderPostingController;
use CloudPrinterPlugin\Controllers\PluginValidateController;
use CloudPrinterPlugin\Controllers\ProductAdminController;
use CloudPrinterPlugin\Controllers\ProductController;
use CloudPrinterPlugin\Controllers\ReorderController;
use CloudPrinterPlugin\Controllers\CloudSignalHandlerController;
use CloudPrinterPlugin\Controllers\DynamicProductController;
use CloudPrinterPlugin\Controllers\DynamicProductOptionsController;
use CloudPrinterPlugin\Controllers\CustomEmailController;
use CloudPrinterPlugin\Controllers\ShippingController;
use CloudPrinterPlugin\Controllers\IncludeSalesCostController;
use CloudPrinterPlugin\Services\Assets\AssetsService;
use CloudPrinterPlugin\Services\Schedule\ScheduleService;

/**
 * Class CloudPrinter
 *
 * @copyright   2019 by CloudPrinter
 * @author      Vasyl Kushniruk <vk@cloudprinter.com>
 */
class CloudPrinter
{
    const DOMAIN = 'cloudprinter';

    private static $_instance = null;

    /**
     * CloudPrinter constructor.
     */
    public function __construct()
    {
        $this->defineConstants();
        $this->initControllers();

    }

    public static function instance()
    {
        if (is_null(self::$_instance)) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }

    private function initControllers()
    {
        add_action( 'plugins_loaded', [$this, 'pluginIsLoaded']);
        new InstallPluginController();
    }

    /**
     * Autoload plugin controllers
     */
    public function pluginIsLoaded() {
        new OrderPostingController();
        new OrderCancelController();
        new CloudSignalHandlerController();
        new ReorderController();
        new ProductAdminController();
        new CustomEmailController();
        new ShippingController();
        new OrderDetailController();
        new IncludeSalesCostController();
        new FixNonceController();
        new ManualProductFileUploadController();

        if (is_admin()) {
            new GeneralSettingsController();
            new OrderAdminController();
            new PluginValidateController();

        } else {
            new CheckoutController();
            new ProductController();
            new DynamicProductController();
            new DynamicProductOptionsController();
        }

        // Scheduled events.
        $cronService = new ScheduleService();
        $cronService->activate();

        // Register all css/js
        $assetService = new AssetsService();
        $assetService->register();

        // Generate nonce
        add_action('init', [$this, 'generateNonce']);
    }

    /**
     * Define CP Constants.
     */
    private function defineConstants()
    {
        $this->define('CP_ABS_PATH', dirname(CP_PLUGIN_FILE) . '/');
        $this->define('CP_PLUGIN_BASENAME', plugin_basename(CP_PLUGIN_FILE));
        $this->define('CP_PLUGIN_URL', untrailingslashit( plugins_url( '/', CP_PLUGIN_FILE )));
    }

    /**
     * Define constant if not already set.
     * @param string $name Constant name.
     * @param string|bool $value Constant value.
     */
    private function define($name, $value)
    {
        if (!defined($name)) {
            define($name, $value);
        }
    }

    public function generateNonce()
    {
        wp_register_script( 'nonce-script', CP_PLUGIN_URL . '/assets/js/cp.js' );
        $variables = ['nonce' => wp_create_nonce( 'wp_rest' )];
        wp_localize_script( 'nonce-script', 'cp', $variables );
        wp_enqueue_script( 'nonce-script' );
    }

}
