<?php

namespace CloudPrinterPlugin\Controllers;

use CloudPrinterPlugin\CloudPrinter;
use CloudPrinterPlugin\Helpers\{CloudPrinterHelper, ConfigHelper, ProductValidationHelper};
use CloudPrinterPlugin\Libs\BaseController;
use CloudPrinterPlugin\Models\CPProduct;
use CloudPrinterPlugin\Services\ProductDesigner\ProductDesignerService;

/**
 * Class ProductDataTabController
 *
 * @copyright   2019 by CloudPrinter
 * @author      Vasyl Kushniruk <vk@cloudprinter.com>
 */
class ProductAdminController extends BaseController
{
    public function __construct()
    {
        parent::__construct(CP_ABS_PATH, 'product-admin');

        //Add custom Cloudprinter.com tab for admin product page
        add_filter('woocommerce_product_data_tabs', [$this, 'customProductTab']);

        // Add custom product option (Cloudprinter.com product)
        add_filter('product_type_options', [$this, 'addCustomProductOption']);

        // Add data panel for custom tab(Cloudprinter.com)
        add_action('woocommerce_product_data_panels', [$this, 'addCustomDataPanel']);

        // Process product custom fields
        add_action('woocommerce_process_product_meta', [$this, 'processSavingCloudPrinterMetaData']);

        // Add enctype="multipart/form-data" for form
        add_action('post_edit_form_tag', [$this, 'addEncTypeForForm']);

        // Add custom column(Cloudprinter) to product list
        add_filter('manage_edit-product_columns', [$this, 'addCustomColumn']);

        // Set value for custom column(Cloudprinter) in product list
        add_action('manage_product_posts_custom_column', [$this, 'setValueForCustomColumn'], 10, 2);

        add_action('admin_notices', [$this, 'productValidation']);

        // Get product designer projects list.
        add_action('rest_api_init', function () {
            register_rest_route('cloudprinter/v1', '/product-designer/projects', [
                'methods' => 'POST',
                'callback' => [$this, 'getProductDesignerProjects'],
                'permission_callback' => [$this, 'verifyApiRequest']
            ]);
        });
    }

    /**
     * Verify rest api request.
     * @return bool
     */
    public function verifyApiRequest()
    {
        $headers = array_change_key_case(getallheaders());

        return wp_verify_nonce($headers['x-wp-nonce'], 'wp_rest') && current_user_can( 'edit_posts' );
    }

    /**
     * Get projects of selected product designer.
     * @param $postData
     * @return array
     */
    public function getProductDesignerProjects($postData)
    {
        $productDesignerAlias = $postData['product_designer_alias'];
        $productDesignerService = new ProductDesignerService();
        $productDesigner = $productDesignerService->getProductDesignerByAlias($productDesignerAlias);

        return $productDesigner->getProjects();
    }

    /**
     * Custom columns in product/list table.
     * @param $columns
     * @return mixed
     */
    public function addCustomColumn($columns)
    {
        return array_slice( $columns, 0, 3, true )
            + ['cloud_printer_product' => __( 'CloudPrinter')]
            + array_slice( $columns, 3, NULL, true );
    }

    /**
     * Set value custom column(Cloudprinter) to product list
     * @param $column
     * @param $postId
     */
    public function setValueForCustomColumn($column, $postId)
    {
        if ($column == 'cloud_printer_product') {
            echo get_post_meta($postId, '_cp_is_cloud_printer_product', true);
        }
    }

    /**
     * Add enctype for product from.
     */
    public function addEncTypeForForm()
    {
        echo ' enctype="multipart/form-data"';
    }

    /**
     * Add custom product option (Cloudprinter.com product)
     * @param $productTypeOptions array
     * @return mixed
     */
    public function addCustomProductOption($productTypeOptions)
    {
        $productTypeOptions['cp_is_cloud_printer_product'] = [
            'id' => '_cp_is_cloud_printer_product',
            'label' => __('Cloudprinter.com product', CloudPrinter::DOMAIN),
            'description' => __('This product related to Cloudprinter.com product.', CloudPrinter::DOMAIN),
            'default' => 'no'
        ];

        return $productTypeOptions;
    }

    /**
     * Add custom tab
     * @param $allProductTabs array
     * @return mixed
     */
    public function customProductTab($tabs)
    {
        $pluginVersion = ConfigHelper::getPluginVersion();
        wp_enqueue_style('cloud-printer-product', CP_PLUGIN_URL . '/assets/css/cloud-printer-product.css', array(), $pluginVersion);
        wp_enqueue_script('cloud-printer-product-js', CP_PLUGIN_URL . '/assets/js/cloud-printer-product.js', ['jquery'], $pluginVersion);
        wp_enqueue_script('cloud-printer-price-check.js', CP_PLUGIN_URL . '/assets/js/cloud-printer-price-check.js', ['jquery'], $pluginVersion);
        wp_enqueue_script('cloud-printer-product-meta-data.js', CP_PLUGIN_URL . '/assets/js/cloud-printer-product-meta-data.js', ['jquery'], $pluginVersion);

        $newTab = [
            "cloud-printer" => [
                'label' => 'Cloudprinter.com',
                'target' => 'wk_cloudprinter_tab_data',
                'priority' => 100000,
                'class' => ['show_if_cloud_printer']
            ]
        ];

        return array_merge($tabs, $newTab);
    }

    /**
     * Save product custom fields
     * @param $postId
     */
    public function processSavingCloudPrinterMetaData($postId)
    {
        $isCloudPrinterProduct = isset($_POST['_cp_is_cloud_printer_product']) ? 'yes' : 'no';

        if ($isCloudPrinterProduct !== 'yes') {
            update_post_meta($postId, '_cp_is_cloud_printer_product', $isCloudPrinterProduct);

            return false;
        }

        $cpProduct = new CPProduct($postId);

        $productOptions = isset($_POST['cp_product_options']) ? serialize($_POST['cp_product_options']) : '';
        $dynamicProductOptions = isset($_POST['cp_dynamic_product_options']) ? serialize($_POST['cp_dynamic_product_options']) : '';


        if ($_POST['cp_product_reference'] && $_POST['cp_product_reference'] !=  get_post_meta($postId, 'cp_product_reference', true)) {
            $productInfo = CloudPrinterHelper::getCpProductInfo($_POST['cp_product_reference']);
            update_post_meta($postId, 'cp_product_info', serialize($productInfo));
        }

        // Product base info fields
        update_post_meta($postId, '_cp_is_cloud_printer_product', $isCloudPrinterProduct);
        update_post_meta($postId, 'cp_product_options', $productOptions);
        update_post_meta($postId, 'cp_product_category', $_POST['cp_product_category']);
        update_post_meta($postId, 'cp_product_reference', $_POST['cp_product_reference']);
        update_post_meta($postId, 'cp_book_count_pages', $_POST['cp_book_count_pages']);

        // Quantity
        if (isset($_POST['cp_quantity_type'])) {
            update_post_meta($postId, 'cp_quantity_type', $_POST['cp_quantity_type']);
        }

        update_post_meta($postId, 'cp_count_pieces', $_POST['cp_count_pieces']);
        update_post_meta($postId, 'cp_product_file_required', $_POST['cp_product_file_required']);

        // File storage type (upload / link)
        update_post_meta($postId, 'cp_file_storage', $_POST['cp_file_storage']);

        // Product option type (fixed / dynamic)
        update_post_meta($postId, 'cp_product_options_type', $_POST['cp_product_options_type']);

        // Dynamic product options
        if ($dynamicProductOptions) {
            update_post_meta($postId, 'cp_dynamic_product_options', $dynamicProductOptions);
        }

        // Product file method.
        update_post_meta($postId, 'cp_file_method', $_POST['cp_file_method']);

        // Product designer.
        if (isset($_POST['cp_product_designer']))
        {
            update_post_meta($postId, 'cp_product_designer', $_POST['cp_product_designer']);
        }
        if (isset($_POST['cp_product_designer_project'])) {
            update_post_meta($postId, 'cp_product_designer_project', $_POST['cp_product_designer_project']);
        }

        $productFiles = $cpProduct->getProductFiles();

        // Uploaded files
        if ($_FILES['cp_file_upload']['name']) {
            foreach ($_FILES['cp_file_upload']['name'] as $fileType => $name) {
                if ($_FILES['cp_file_upload']['tmp_name'][$fileType]) {
                    $fileContent = file_get_contents($_FILES['cp_file_upload']['tmp_name'][$fileType]);
                    $extension = array_reverse(explode(".", $name))[0];
                    $newName = md5($name . time()) . '.' .$extension;
                    $uploaded = wp_upload_bits($newName, null, $fileContent);
                    $uploaded['md5'] = md5($fileContent);
                    $uploaded['name'] = $name;

                    if (!empty($uploaded)) {
                        $productFiles['local'][$fileType] = $uploaded;
                    }
                }
            }
        }

        // Save the external file links.
        if (!empty($_POST['cp_file_url'])) {
            foreach ($_POST['cp_file_url'] as $fileType => $url) {
                $productFiles['external'][$fileType]['link'] = $url;
            }
        }

        // Save the external file md5.
        if (!empty($_POST['cp_file_url_md5'])) {
            foreach ($_POST['cp_file_url_md5'] as $fileType => $md5) {
                $productFiles['external'][$fileType]['md5'] = $md5;
            }
        }

        $cpProduct->saveProductFiles($productFiles);
    }

    /**
     * Add custom data panel for tab(Cloudprinter.com)
     */
    public function addCustomDataPanel()
    {
        global $post;
        $countries = CloudPrinterHelper::getCountries();
        $apiKey = get_option('cp_api_key');
        $dynamicProductStorage = get_option('cp_dynamic_product_storage');
        $dropboxAccessToken = get_option('cp_dropbox_access_token');

        $cpProduct = new CPProduct($post->ID);
        $productFiles = $cpProduct->getProductFiles();



        // Product option type (fixed / dynamic)
        $productOptionsType = get_post_meta($post->ID, 'cp_product_options_type', true);

        // Dynamic product options
        $dynamicProductOptions = get_post_meta($post->ID, 'cp_dynamic_product_options', true);

        // File storage type (upload /set link)
        $fileStorage = get_post_meta($post->ID, 'cp_file_storage', true);

        // Product file method.
        $fileMethod = get_post_meta($post->ID, 'cp_file_method', true);

        // Quantity
        $quantityType = get_post_meta($post->ID, 'cp_quantity_type', true);
        $countPieces = get_post_meta($post->ID, 'cp_count_pieces', true);

        // Max upload size
        $maxUpload = $this->getMaxUploadSize();

        // Product designers
        $productDesignersService = new ProductDesignerService();
        $productDesigners = $productDesignersService->getProductDesignerEditors();
        $productDesigner = get_post_meta($post->ID, 'cp_product_designer', true);
        $productDesignerProject = get_post_meta($post->ID, 'cp_product_designer_project', true);


        // Render cp product settings view.
        $this->render('cloud-printer-tab', [
            'post' => $post,
            'apiKey' => $apiKey,
            'fileStorage' => $fileStorage,
            'productOptionsType' => $productOptionsType,
            'productFiles' => $productFiles,
            'maxUpload' => $maxUpload,
            'quantityType' => $quantityType,
            'countPieces' => $countPieces,
            'fileMethod' => $fileMethod,
            'dynamicProductStorage' => $dynamicProductStorage,
            'dropboxAccessToken' => $dropboxAccessToken,
            'productDesigners' => $productDesigners,
            'productDesigner' => $productDesigner,
            'productDesignerProject' => $productDesignerProject
        ]);

        $this->render('price-check', [
            'countries' => $countries
        ]);
    }

    /**
     * Product validation.
     */
    public function productValidation() {
        /** @var $product \WC_Product */
        global $post;
        global $pagenow;

        if ($post && $post->post_type == 'product' && in_array($pagenow, ['post.php', 'post-new.php'])) {
            try {
                ProductValidationHelper::getInstance()->validate($post->ID);
            } catch (\Exception $e) {
                $class = 'notice notice-error';
                $message = 'Cloudprinter.com product configured wrong. ' . $e->getMessage();

                printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), esc_html($message));
            }
        }
    }

    private function getMaxUploadSize()
    {
        $maxUpload = min(ini_get('post_max_size'), ini_get('upload_max_filesize'));

        return str_replace('M', 'MB', $maxUpload);
    }
}
