<?php

namespace CloudPrinterPlugin\Services\CustomEmail;

/**
 * Class PriceMarginAlertEmail
 *
 * @copyright 2020 by CloudPrinter
 * @author    Vasyl Kushniruk <vk@cloudprinter.com>
 */
class PriceMarginAlertEmail extends \WC_Email
{
    private $printProducts;
    private $percentageChange;
    private $cloudprinterProductsPrice;

    public function __construct()
    {
        $this->id = 'wc_cloudprinter_price_margin_alert';
        $this->customer_email = false;
        $this->title = 'Cloudprinter.com revenue check alert';
        $this->description = 'Emails are sent when the items selling below a certain margin level';
        $this->heading = 'The items are selling below a certain margin level';
        $this->subject = 'The items are selling below a certain margin level';

        // template path
        $this->template_base = CP_ABS_PATH . 'src/views/emails/';
        $this->template_html = 'price_margin_alert.php';
        $this->recipient = $this->get_option('recipient', get_option('admin_email'));

        // Trigger from revenue check
        add_action('cloudprinter_price_margin_alert', [$this, 'trigger'], 10, 3);
        parent::__construct();
    }


    /**
     * Determine if the email should actually be sent and setup email merge variables
     */
    public function trigger($percentageChange, $printProducts, $cloudprinterProductsPrice)
    {
        if (!$this->is_enabled() || !$this->get_recipient()) {
            return;
        }

        $this->percentageChange = $percentageChange;
        $this->printProducts = $printProducts;
        $this->cloudprinterProductsPrice = $cloudprinterProductsPrice;

        //Send the email!
        $this->send($this->get_recipient(), $this->get_subject(), $this->get_content(), $this->get_headers(), $this->get_attachments());
    }

    private function getTrackingCodesLines($emailShippingData)
    {
        $trackingLines = [];
        $shipmentNumber = 1;
        foreach ($emailShippingData as $shippingOption => $shippingItems) {
            foreach ($shippingItems as $shippingItem) {
                $line = '';
                $trackingCode = !empty($shippingItem['tracking_code']) ? ' with tracking ' . $shippingItem['tracking_code'] : '';
                $line .= '<b>Shipment ' . $shipmentNumber . ': </b><br>' . $shippingOption . $trackingCode . '<br>';
                $line .= join('<br>', $shippingItem['product_name']);

                if ($shippingItem['tracking_url']) {
                    $line .= '<br> <a href="' . $shippingItem['tracking_url'] . '" target="_blank">Click here for tracking information.</a>';
                }

                $trackingLines[] = $line . '<br>';
                $shipmentNumber++;
            }
        }
        return $trackingLines;
    }

    /**
     * get_content_html function.
     *
     * @return string
     * @since 0.1
     */
    public function get_content_html()
    {
        ob_start();

        extract([
            'email_heading' => $this->get_heading(),
            'printProducts' => $this->printProducts,
            'percentageChange' => $this->percentageChange,
            'cloudprinterProductsPrice' => $this->cloudprinterProductsPrice,
        ]);
        $emailTemplatePath = $this->template_base . $this->template_html;
        include $emailTemplatePath;

        return ob_get_clean();
    }

    /**
     * get_content_plain function.
     *
     * @return string
     * @since 0.1
     */
    public function get_content_plain()
    {
        ob_start();
        woocommerce_get_template($this->template_plain, [
            'email_heading' => $this->get_heading()
        ]);

        return ob_get_clean();
    }


    /**
     * Initialize Settings Form Fields
     *
     * @since 2.0
     */
    public function init_form_fields()
    {

        $placeholder_text = sprintf(__('Available placeholders: %s', 'woocommerce'), '<code>' . esc_html(implode('</code>, <code>', array_merge(array_keys($this->placeholders), ['{order_number}']))) . '</code>');
        $this->form_fields = array(
            'enabled' => array(
                'title' => 'Enable/Disable',
                'type' => 'checkbox',
                'label' => 'Enable this email notification',
                'default' => 'yes'
            ),
            'recipient' => array(
                'title' => __('Recipient(s)', 'woocommerce'),
                'type' => 'text',
                /* translators: %s: WP admin email */
                'description' => sprintf(__('Enter recipients (comma separated) for this email. Defaults to %s.', 'woocommerce'), '<code>' . esc_attr(get_option('admin_email')) . '</code>'),
                'placeholder' => '',
                'default' => get_option('admin_email'),
                'desc_tip' => true,
            ),
            'subject' => array(
                'title' => 'Subject',
                'type' => 'text',
                'description' => 'This controls the email subject line. ' . $placeholder_text,
                'placeholder' => '',
                'desc_tip' => true,
                'default' => $this->subject
            ),
            'heading' => array(
                'title' => 'Email Heading',
                'type' => 'text',
                'description' => __('This controls the main heading contained within the email notification.') . $placeholder_text,
                'placeholder' => '',
                'default' => $this->heading,
                'desc_tip' => true,
            ),

            'email_type' => array(
                'title' => 'Email type',
                'type' => 'select',
                'description' => 'Choose which format of email to send.',
                'default' => 'html',
                'class' => 'email_type',
                'options' => array(
                    'plain' => __('Plain text', 'woocommerce'),
                    'html' => __('HTML', 'woocommerce'),
                    'multipart' => __('Multipart', 'woocommerce'),
                )
            )
        );
    }
}
