<?php

namespace CloudPrinterPlugin\Services\Storage;

use mysql_xdevapi\Exception;

/**
 * @copyright 2020 by CloudPrinter
 * @author    Vasyl Kushniruk <vk@cloudprinter.com>
 */

class DropboxService implements StorageInterface
{
    /**
     * API URLs
     */
    const DROPBOX_API_V2_BASE_URL = 'https://api.dropboxapi.com/2';
    const DROPBOX_UPLOAD_HOST = 'https://content.dropboxapi.com/2';

    /**
     * Dropbox folders.
     */
    const TEMPORARY_FOLDER_NAME = 'uploading';
    const ORDERED_FOLDER_NAME = 'ordered';

    private $accessToken = null;

    public function __construct($accessToken = null)
    {
        $this->accessToken = $accessToken;
        if (!$this->accessToken) {
            $this->accessToken = $this->getAccessToken();
        }
    }

    /**
     * Get username of the current dropbox user
     * @return array
     */
    public function getCurrentUsername()
    {
        $currentAccountInfo = $this->makeRequest('/users/get_current_account');

        if ($currentAccountInfo) {
            return $currentAccountInfo['name']['display_name'];
        }

        return false;
    }

    /**
     * Get information about dropbox storage free space.
     * @return array
     */
    public function getStorageInfo()
    {
        $result = $this->makeRequest('/users/get_space_usage');

        return $result;
    }

    /**
     * Upload file to dropbox, to temporary folder.
     * @param $file
     * @return array
     * @throws \Exception
     */
    public function uploadFileToTemporaryFolder($file)
    {
        $result = $this->makeUploadRequest('/files/upload', $file);

        if (isset($result['name'])) {
            return $result['name'];
        }

        throw new \Exception('DropboxService::uploadFileToTemporaryFolder: The file name is empty');
    }

    /**
     * Upload file to dropbox, to temporary folder.
     * @param $file
     * @return array
     * @throws \Exception
     */
    public function uploadFileToOrderedFolder($file)
    {
        $result = $this->makeUploadRequest('/files/' . ORDERED_FOLDER_NAME, $file);

        if (isset($result['name'])) {
            return $result['name'];
        }

        throw new \Exception('DropboxService::uploadFileToTemporaryFolder: The file name is empty');
    }

    /**
     * Remove the file from temporary folder
     * @param $fileName
     * @return bool
     */
    public function removeFileFromTemporaryFolder($fileName)
    {
        $data = [
            'path' => '/' . self::TEMPORARY_FOLDER_NAME . '/' . $fileName
        ];

        return $this->makeRequest('/files/delete_v2', $data);
    }

    /**
     * Move files from tmp folder to ordered.
     * @param $files
     * @return bool
     */
    public function moveFilesFromTmpFolderToOrdered($files)
    {
        $data = ['entries' => []];

        foreach ($files as $fileName) {
            $data['entries'][] = [
              'from_path' => '/' . self::TEMPORARY_FOLDER_NAME . '/' . $fileName,
              'to_path' => '/' . self::ORDERED_FOLDER_NAME . '/' . $fileName
            ];
        }

        return $this->makeRequest('/files/move_batch_v2', $data);
    }

    /**
     * Get public shared link for file.
     * @param $fileName
     * @return string
     */
    public function getSharedLink($fileName)
    {
        $data = ['path' => '/' . self::ORDERED_FOLDER_NAME . '/' . $fileName];

        $response = $this->makeRequest('/sharing/create_shared_link', $data);
        if (!empty($response['url'])) {
            return str_replace('dl=0', 'dl=1', $response['url']);
        }

        return false;
    }

    /**
     * Revoke the public shared link
     * @param $url
     * @return bool|mixed
     */
    public function revokeSharedLink($url)
    {
        $data = ['url' => $url];
        return $this->makeRequest('/sharing/revoke_shared_link', $data);
    }

    /**
     * Remove list of files.
     * @param $files array
     * @return bool
     */
    public function removeFilesFromOrderedFolder($files)
    {
        $entries = [];
        foreach ($files as $file) {
            $entries[] = ["path" => '/' . self::ORDERED_FOLDER_NAME . '/' . $file];
        }

        $data = [
            "entries" => $entries
        ];

        return $this->makeRequest('/files/delete_batch', $data);
    }

    /**
     * Get the temporary link for the file. Expiration time: in 4 hours.
     * @param $fileName
     * @return string
     */
    public function getTemporaryLink($fileName)
    {
        $data = [
            "path" => '/' . self::ORDERED_FOLDER_NAME . '/' . $fileName
        ];
        $result = $this->makeRequest('/files/get_temporary_link', $data);

        if (!empty($result['link'])) {
            return $result['link'];
        }

        return false;
    }

    /**
     * Get current dropbox access token.
     * @return mixed
     */
    private function getAccessToken()
    {
        return get_option('cp_dropbox_access_token');
    }

    /**
     * @param $endpoint
     * @param string|array $data
     * @return array
     */
    private function makeRequest($endpoint, $data = 'null')
    {
        $headers = [
            "Authorization: Bearer " . $this->accessToken,
            "Content-Type: application/json"
        ];

        if (is_array($data))
        {
            $data = json_encode($data);
        }

        $ch = curl_init(self:: DROPBOX_API_V2_BASE_URL . $endpoint);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        if (in_array($httpCode, [200, 201]) && $response) {
            return json_decode($response, true);
        } else {
            throw new \Exception($response);
        }
    }

    /**
     * @throws \Exception
     */
    private function makeUploadRequest($endpoint, $file)
    {
        $extension = array_reverse(explode(".", $file['file']['name']))[0];
        $headers = [
            "Authorization: Bearer " . $this->accessToken,
            'Content-Type: application/octet-stream',
            'Dropbox-API-Arg: {"path":"/' . self::TEMPORARY_FOLDER_NAME . '/'.md5(rand(0, 100000000) + time()).'.' . $extension . '", "mode":"add"}'
        ];

        $ch = curl_init(self:: DROPBOX_UPLOAD_HOST . $endpoint);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_PUT, true);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $fp = fopen($file['file']['tmp_name'], 'rb');
        curl_setopt($ch, CURLOPT_INFILE, $fp);
        curl_setopt($ch, CURLOPT_INFILESIZE, $file['file']['size']);
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        if (in_array($httpCode, [200, 201]) && $response) {
            return json_decode($response, true);
        } else {
            throw new \Exception($response);
        }
    }
}
